# copyright (C) 1997-98 Jean-Luc Fontaine (mailto:jfontain@mygale.org)
# this program is free software: please refer to the BSD type license enclosed in this package

set rcsId {$Id: datagraf.tcl,v 1.37 1998/05/05 21:12:11 jfontain Exp $}

class dataGraph {

if {$::officialBLT} {
    set (widget) stripchart
} else {
    set (widget) graph
}

    proc dataGraph {this parentPath args} composite {
        [new $dataGraph::(widget) $parentPath -title {} -topmargin 3 -bufferelements 0 -plotborderwidth 1] $args
    } blt2DViewer {$widget::($this,path)} {
        $widget::($this,path) xaxis configure -tickfont $font::(smallNormal) -title {} -rotate 90 -command dataGraph::axisTime
if {$::officialBLT} {
        $widget::($this,path) xaxis configure -tickshadow {}
}
        # track size updates (do not use Configure event since it leads to incorrect values for graph internal components)
        bind $widget::($this,path) <Visibility> "dataGraph::resized $this"
        set dataGraph::($this,plotWidth) 0                                                                       ;# cache plot width
        composite::complete $this
    }

    proc ~dataGraph {this} {}

    proc iconData {} {
        return {
            R0lGODdhKAAoAKUAAHt5e87PzgAAANbX1v///zFhITFhGJTXa633e6X3e5znc6Xvc3u+Wpznazk4
            OZTfa4zPY3NJAPffSufHOda2Mc6mKVIIY2sYexBhe4wQpa3X54S+zr0o1qUYvbUYzrUg1lKWrb0Y
            3t44/60Yzt5B/+dJ/+dR/9YY984Y7+dZ/+dh/94o/+9x/wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
            AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwAAAAAKAAoAAAG/kCAcEgsGo/IYQAg
            aDqf0Kh02lwKAthsYIDlbrVer3Y8ZgYI6DQaoG6z2+n3Gm09wwlyeN6tflsHd3iBgoF5fmZDBYpJ
            AIoGBUaPj0Z1aAUHmAWEapeYB5ppBQgICQqghwKABJ2ZeYoLCwoLpYqKpbAMkISVqwyeC6CrsAoK
            DAzDCg3FC8aQbKhLQgW+BwsNtdUNzdMMxb7bDuFEf5zGyp4H2osOisbbAA5y0G3cD98QuUPxjbX6
            fXRMVHHi5s7ZHCNz4gC8gqTWIkYQAfCCs2/Iv0IXyd2JwJEjH4wKCUxMw1GCSZMRLt4xtDDakAgm
            J8iUKSGCkY42GY2EKZNC/oWfFCbUTAmg5EmPLEUGJCmhZ4WOFYKihNl05tSMZtDwnOAzZ9GfNJv6
            /BlVaM2Qf4REoMD2abi3HKOy7Qo1aNeKErMSiEDWpgM3fPsSCfzT5i69ezkK+Qu4oyGcQg5f6ROZ
            sZ54K/cltEKls2cpViyIvoCh9IXTqFOXNp2a9OrWF6xkmI1Bg20Ms3Pnrm1bA+7dvTdguMChOAcr
            HTx8wLChue8P0KGX1tDc+era1UEMDxFChAgrIzyMwACi+nPptpuDKL+hN/UN64dfCEGiPvgR0tm3
            X/0+Pnn98GknH3clFAgeByEg+J9z6cHHWmnr+Ycad/XZxwR3GIawYHUOXsJ23YQYmiCiCVZwhyCC
            Gv4X4YAYwjbiiyOWeAIKM6KAQgiuvXZBCjz26OOPQFpRo41E4phaBiGooOSSTDbppJAoOCnllFQ2
            acUKK7Cg5ZZcdunll18uQcaYZJZZZhAAOw==
        }
    }

    proc options {this} {
        # force size and interval values
        return [list\
            [list -height height Height 200]\
            [list -interval interval Interval 5]\
            [list -samples samples Samples 101 101]\
            [list -width width Width 300]\
        ]
    }

    foreach option {-height -width} {
        proc set$option {this value} "\$widget::(\$this,path) configure $option \$value"
    }

    proc set-interval {this value} {
        updateGraduations $this
    }

    proc set-samples {this value} {                                                                     ;# stored at composite level
        if {$composite::($this,complete)} {
            error {option -samples cannot be set dynamically}
        }
    }

if {!$::officialBLT} {
    proc xAxisUpdate {this currentTime} {
        $widget::($this,path) xaxis configure\
            -min [expr {$currentTime-($composite::($this,-samples)*$composite::($this,-interval))}] -max $currentTime
    }
}

    proc axisTime {path value} {
        return [clock format $value -format %H:%M]                                    ;# do not show seconds as they are always zero
    }

    proc newElement {this path args} {                                                          ;# invoked from 2D viewer base class
        return [eval new element $path $args]
    }

if {!$::officialBLT} {
    proc updateTimeDisplay {this seconds} {
        xAxisUpdate $this $seconds
    }
}

    proc updateElement {this element seconds value} {
        element::update $element $seconds $value
    }

    proc updateGraduations {this} {
        if {$dataGraph::($this,plotWidth)==0} return                                                  ;# plot width is not known yet
        set range [expr {$composite::($this,-samples)*$composite::($this,-interval)}]
        # number=$division*($range/$step)                                                                      # number of divisions
        # ($plotWidth/$number)>2                              # make sure that there is at least 1 pixel between neighbour divisions
        # => $step>((2*$division*$range)/$plotWidth)
        set minimum [expr {(2*6*$range)/$dataGraph::($this,plotWidth)}]       ;# use the maximum division in divisions discrete list
        # choose among predefined discrete values
        foreach step {60 300 600 1800 3600 18000 36000 86400} division {6 5 5 5 6 5 5 4} {
            if {$step>$minimum} break
        }
        $widget::($this,path) xaxis configure -stepsize $step -subdivisions $division
if {$::officialBLT} {
        $widget::($this,path) xaxis configure -range $range -shiftby $composite::($this,-interval)
} else {
        xAxisUpdate $this [clock seconds]
}
    }

    proc resized {this} {
        set width [$widget::($this,path) extents plotwidth]
        if {$width!=$dataGraph::($this,plotWidth)} {
            set dataGraph::($this,plotWidth) $width
            updateGraduations $this                                                       ;# optimize graduations for new plot width
        }
    }

}

class dataGraph {

    class element {

if {$::officialBLT} {
        set (vectorIndex) 0
}

        proc element {this path args} switched {$args} {
            $path element create $this -label {} -symbol none                         ;# use object identifier as element identifier

            set dots [expr {$configuration::(graphNumberOfIntervals)+1}]
if {$::officialBLT} {
            ### blt::vector create #auto($dots) should ultimately work ###
            set ($this,xVector) [blt::vector create vector[incr (vectorIndex)]($dots)]                         ;# x axis data vector
            set ($this,yVector) [blt::vector create vector[incr (vectorIndex)]($dots)]                         ;# y axis data vector
} else {
            set ($this,xVector) xVector${this}
            set ($this,yVector) yVector${this}
            global [set ($this,xVector)] [set ($this,yVector)]
            blt::vector [set ($this,xVector)]($dots)
            blt::vector [set ($this,yVector)]($dots)
}
            $path element configure $this -xdata [set ($this,xVector)] -ydata [set ($this,yVector)]

            set ($this,path) $path
            switched::complete $this
        }

        proc ~element {this} {
if {$::officialBLT} {
            blt::vector destroy [set ($this,xVector)] [set ($this,yVector)]
} else {
            global [set ($this,xVector)] [set ($this,yVector)]
            unset [set ($this,xVector)] [set ($this,yVector)]
}
            [set ($this,path)] element delete $this
            if {[string length $switched::($this,-deletecommand)]>0} {
                uplevel #0 $switched::($this,-deletecommand)                                ;# always invoke command at global level
            }
        }

        proc options {this} {
            return [list\
                [list -color black black]\
                [list -deletecommand {} {}]\
                [list -label {} {}]\
            ]
        }

        foreach option {-color -label} {
            proc set$option {this value} "\[set (\$this,path)\] element configure \$this $option \$value"
        }

        proc set-deletecommand {this value} {}                                                   ;# data is stored at switched level

        proc update {this x y} {
if {!$::officialBLT} {
            global [set ($this,xVector)] [set ($this,yVector)]
}
            if {[string length $y]==0} {
                set y 0                                                              ;# too bad BLT graph cannot handle empty values
                [set ($this,path)] element configure $this -label "$switched::($this,-label): ?"
            } else {
                [set ($this,path)] element configure $this -label "$switched::($this,-label): $y"
            }
            [set ($this,xVector)] delete 0                                            ;# achieve scrolling by deleting first element
            [set ($this,yVector)] delete 0
            [set ($this,xVector)] append $x                                                               ;# and appending new value
            [set ($this,yVector)] append $y
        }
    }
}
